# -*- coding: utf-8 -*-
#
#  filters.py
#  collar
#

"""
Basic key-value filtering.
"""

from . import errors


def compile(criteria):
    filters = [compile_spec(s) for s in criteria]
    return lambda i: all(f(i) for f in filters)


class InvalidCriteria(errors.UserError):
    pass


def compile_spec(spec):
    try:
        k, v = spec.split(':', 1)
    except ValueError:
        raise InvalidCriteria('invalid search criteria: {}'.format(spec))

    filter_f = lambda i: (v in i[k]
                          if isinstance(i.get(k), list)
                          else i.get(k) == v)
    return filter_f
