# -*- coding: utf-8 -*-
#
#  fleet.py
#  collar
#

"""
Interaction with EC2.
"""

from operator import itemgetter

from boto import ec2

from . import errors


class InvalidRegion(errors.UserError):
    pass


class Fleet(object):
    "An entire EC2 server set for an account."
    def __init__(self, region, instances=None):
        self.conn = ec2.connect_to_region(region)
        if not self.conn:
            raise InvalidRegion('invalid region: {}'.format(region))

        self.instances = instances or self._load_instances(self.conn)

    def _load_instances(self, conn):
        instances = []
        for reservation in conn.get_all_instances():
            for instance in reservation.instances:
                instances.append(instance_to_dict(instance))

        instances.sort(key=itemgetter('Id'))

        return instances

    def __iter__(self):
        return iter(self.instances)

    def __len__(self):
        return len(self.instances)


def instance_to_dict(i):
    "Move from boto native instance to a vanilla dictionary."
    summary = i.tags.copy()
    summary['Id'] = i.id
    summary['InstanceType'] = i.instance_type
    summary['KeyName'] = i.key_name
    summary['PrivateDnsName'] = i.private_dns_name
    summary['PublicDnsName'] = i.public_dns_name or None
    summary['PrivateIPAddress'] = i.private_ip_address
    summary['PublicIpAddress'] = i.ip_address or None
    summary['State'] = i.state
    summary['SecurityGroups'] = sorted(g.id for g in i.groups)
    return summary
